SystemOrganization addCategory: #'BinaryModules-Tests'!
SystemOrganization addCategory: #BinaryModules!
SystemOrganization addCategory: #'BinaryModules-Tests-Data'!

!Morph methodsFor: '*BinaryModules' stamp: 'lw 1/15/2013 22:57'!
addStandardHaloMenuItemsTo: aMenu hand: aHandMorph
	"Add standard halo items to the menu"

	| unlockables |

	self isWorldMorph ifTrue:
		[^ self addWorldHaloMenuItemsTo: aMenu hand: aHandMorph].

	self mustBeBackmost ifFalse:
		[aMenu add: 'send to back' translated action: #goBehind.
		aMenu add: 'bring to front' translated action: #comeToFront.
		self addEmbeddingMenuItemsTo: aMenu hand: aHandMorph.
		aMenu addLine].

	self addFillStyleMenuItems: aMenu hand: aHandMorph.
	self addBorderStyleMenuItems: aMenu hand: aHandMorph.
	self addDropShadowMenuItems: aMenu hand: aHandMorph.
	self addLayoutMenuItems: aMenu hand: aHandMorph.
	self addHaloActionsTo: aMenu.
	owner isTextMorph ifTrue:[self addTextAnchorMenuItems: aMenu hand: aHandMorph].
	aMenu addLine.
	self addToggleItemsToHaloMenu: aMenu.
	aMenu addLine.
	self addCopyItemsTo: aMenu.
	self addPlayerItemsTo: aMenu.
	self addExportMenuItems: aMenu hand: aHandMorph.
	self addStackItemsTo: aMenu.
	self addMiscExtrasTo: aMenu.
	Preferences noviceMode ifFalse:
		[self addDebuggingItemsTo: aMenu hand: aHandMorph].

	aMenu addLine.
	aMenu defaultTarget: self.

	aMenu addLine.

	unlockables := self submorphs select:
		[:m | m isLocked].
	unlockables size = 1 ifTrue:
		[aMenu
			add: ('unlock "{1}"' translated format: unlockables first externalName)
			action: #unlockContents].
	unlockables size > 1 ifTrue:
		[aMenu add: 'unlock all contents' translated action: #unlockContents.
		aMenu add: 'unlock...' translated action: #unlockOneSubpart].
	
	BMPackage add: self toBinaryModuleSubmenuTo: aMenu.
	
	aMenu defaultTarget: aHandMorph.
! !

!MultiByteFileStream methodsFor: '*BinaryModules' stamp: 'lw 12/21/2012 14:52'!
nextPutAll: aCollection

	(self isBinary or: [aCollection class == ByteArray]) ifTrue: [
		^ super nextPutAll: aCollection.
	].
	aCollection class == WordArrayForSegment ifTrue: [
		^ super nextPutAll: (Bitmap withAll: aCollection) asByteArray
	].
	^converter nextPutAll: aCollection toStream: self! !

Object subclass: #BMBundle
	instanceVariableNames: 'classDescriptions serializedObjects name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMBundle commentStamp: 'lw 1/15/2013 23:20' prior: 0!
A BMBundle is a shortliving object. It is meant to be serialized to a file and give documented access to such a file, much like a struct.
It is our choice over a Dictionary with (somewhere) well-defined keys, in hope that is it more transparent ( in the not software library meaning ).
!

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/13/2013 18:25'!
fromFile: aFileName
	| bundle |
	bundle := FLMaterializer materializeFromFileNamed: aFileName.
	^ (bundle isKindOf: self)
		ifTrue: [ bundle ]
		ifFalse: [ nil ]! !

!BMBundle class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 00:50'!
named: aName classDescriptions: someBMClassDescriptions serializedObjects: someBinary
	^self new
		name: aName asString;
		classDescriptions: someBMClassDescriptions;
		serializedObjects: someBinary;
		yourself      ! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:42'!
classDescriptions
	^ classDescriptions! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:43'!
classDescriptions: aCollection
	classDescriptions := aCollection! !

!BMBundle methodsFor: 'accessing' stamp: 'lw 1/15/2013 23:07'!
name
	^name! !

!BMBundle methodsFor: 'accessing' stamp: 'lw 1/15/2013 23:07'!
name: aName
	name := aName! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:42'!
serializedObjects
	^ serializedObjects! !

!BMBundle methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:43'!
serializedObjects: aByteArray
	serializedObjects := aByteArray! !

Object subclass: #BMClassDescription
	instanceVariableNames: 'name instanceVariables category classSideMethods compiledMethods superclass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:08'!
for: aClass
	^ super new
		initializeFrom: aClass;
		yourself! !

!BMClassDescription class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:08'!
new
	^ self shouldNotImplement! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'lw 1/15/2013 23:47'!
initializeFrom: aClass
	
	name := aClass name.
	superclass := aClass superclass name.
	instanceVariables := aClass instVarNames.
	category := aClass category.
	compiledMethods := aClass methodDict collect: [ :each | BMCompiledMethod for: each].
	classSideMethods := aClass class methodDict collect: [ :each | BMCompiledMethod for: each].! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'lw 1/15/2013 23:48'!
install
	| newClass |
	
	(Smalltalk at: name ifAbsent: nil) ifNotNil: [self error: 'Class already exists: ', name asString ].
	newClass := (Smalltalk at: superclass) subclass: name
		instanceVariableNames: (String streamContents: [ :stream | instanceVariables do: [ :each | stream << each <<$ ] ])
		classVariableNames: ''
		poolDictionaries: ''
		category: category.
		
	compiledMethods do: [ :aMethod | aMethod addTo: newClass].
	classSideMethods do: [ :aMethod | aMethod addTo: newClass class].
	
	^newClass! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 10:50'!
name
	^name! !

!BMClassDescription methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 11:23'!
represents: aClass

	instanceVariables = aClass instVarNames ifFalse: [^false].
	category = aClass category ifFalse: [^false].
	compiledMethods size = aClass methodDict size ifFalse: [^false].
	^true! !

Object subclass: #BMCompiledMethod
	instanceVariableNames: 'bytecode protocol source selector headerWord'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMCompiledMethod class methodsFor: 'instance creation' stamp: 'dg 1/10/2013 11:11'!
for: aMethod

	^super new
			initializeFrom: aMethod; 
			yourself! !

!BMCompiledMethod class methodsFor: 'instance creation' stamp: 'dg 1/10/2013 11:09'!
new

	^self shouldNotImplement! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:36'!
addTo: aClass

	| aCompiledMethod |
	aCompiledMethod := CompiledMethod newMethod: (bytecode size - ((self numLiterals + 1) * 4)) 
											header: headerWord.
	aCompiledMethod replaceFrom: 1 to: bytecode size with: bytecode.
	"If interrupted at this point the VM might very likely crash."
	aCompiledMethod methodClass: aClass.
	aClass methodDict at: selector put: aCompiledMethod.
	
	"aCompiledMethod properties at: #source put: source"! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'dg 1/10/2013 15:03'!
initializeFrom: aMethod

	protocol := aMethod methodClass organization categoryOfElement: aMethod selector.
	selector := aMethod selector.
	source := aMethod getSource.
	bytecode := ByteArray newFrom: aMethod.
	headerWord := aMethod header! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:35'!
numLiterals
	"Answer the number of literals used by the receiver."
	"copy from CompiledMethod"
	^ (headerWord bitShift: -9) bitAnd: 16rFF! !

!BMCompiledMethod methodsFor: 'as yet unclassified' stamp: 'lw 1/10/2013 16:23'!
printOn: aStream

	super printOn: aStream.
	aStream << '(#' << selector << ', ' << headerWord << ', ' << bytecode size << ')'! !

Object subclass: #BMInstallation
	instanceVariableNames: 'package classes objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMInstallation commentStamp: 'lw 1/15/2013 22:55' prior: 0!
A BMInstallation is the installed version of a BMPackage. It contains an Array of all Classes belonging to that Package.

Instance Variables
	classes:		<Object>
	name:		<Object>
	objects:		<Object>

classes
	- xxxxx

name
	- xxxxx

objects
	- xxxxx
!

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 22:57'!
installFromFileNamed: aFileName
	^ (BMPackage fromFileNamed: aFileName)
		ifNil: [ self error: 'Error during bundle load' ]
		ifNotNil: [ :bundle | bundle install ]! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 01:10'!
named: name
	^ super new
		name: name asString;
		classes: #();
		objects: #();
		yourself! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/14/2013 00:50'!
named: aString withClasses: classes andObjects: objects
	^ (self named: aString asString)
		classes: classes;
		objects: objects;
		yourself! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'RL 1/9/2013 11:18'!
new
	self shouldNotImplement! !

!BMInstallation class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 23:15'!
of: aPackage with: someClasses and: someObjects
	^super new
		of: aPackage;
		classes: someClasses;
		objects: someObjects;
		yourself! !

!BMInstallation class methodsFor: 'as yet unclassified' stamp: 'WS 1/9/2013 18:19'!
removeFromSystem
	^ self notYetImplemented! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:04'!
addClass: aClass
	
	self assert: aClass isMeta not.
	(classes includes: aClass) ifFalse: 
		[classes := classes copyWith: aClass].
	self changed: #addClass: with: aClass.
	^aClass! !

!BMInstallation methodsFor: 'system integration' stamp: 'WS 1/14/2013 11:26'!
addMorph: aMorph
	
	self inform: 'Remember to also add the classes neccessary to create the morph.'.
	self addObject: aMorph  ! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:04'!
addObject: anObject
	
	(objects includes: anObject) ifFalse:
		[objects := objects copyWith: anObject].
	self changed: #addObject: with: anObject.
	^anObject! !

!BMInstallation methodsFor: 'accessing' stamp: 'WS 1/9/2013 18:02'!
classes
	^ classes! !

!BMInstallation methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:18'!
classes: aCollection
	classes := aCollection.
	self changed: #classes: with: aCollection! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:20'!
exportToFile
	^ self pack exportToFile! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:21'!
exportToFile: aFileName
	^ self pack exportToFile: aFileName! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:21'!
extractToFile: aFileName
	^ self pack extractToFile: aFileName! !

!BMInstallation methodsFor: 'package behavior' stamp: 'lw 1/15/2013 23:21'!
name: aName
	^self pack name: aName! !

!BMInstallation methodsFor: 'accessing' stamp: 'WS 1/9/2013 18:03'!
objects
	^ objects! !

!BMInstallation methodsFor: 'accessing' stamp: 'lw 1/13/2013 18:19'!
objects: aCollection
	objects := aCollection.
	self changed: #objects: with: aCollection! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:16'!
of: aPackage
	self assert: package isNil.
	package := aPackage! !

!BMInstallation methodsFor: 'accessing - derived' stamp: 'lw 1/15/2013 23:20'!
pack
	package ifNil: [package := BMPackage fromInstallation: self].
	^package! !

Object subclass: #BMPackage
	instanceVariableNames: 'installation bundle name'
	classVariableNames: 'BMBundles'
	poolDictionaries: ''
	category: 'BinaryModules'!

!BMPackage commentStamp: 'lw 1/15/2013 22:56' prior: 0!
A BMPackage is a Stub enabling to find and install BMInstallations.

Instance Variables
	file:		<Object>
	name:		<Object>
	package:		<Object>

file
	Either an object which understands name, classeDescriptions and serializedObjects, or a String, which points to a file where we can find these information.
	We use the file to see that this bundle has been saved, and is not changed since then.
	Saving the filename instead of the struct, avoids saving the same information in two places in the image. In case of class-descriptions, that might not be much overhead, but they include CompiledMethod-Descriptions, with source, and serializedObjects size is unknown.

name
	A String used to find me. Should be the same as used in BMPackage and is copied to BMFile for serialization.

installation
	If the package is installed, this variable points to the BMINstallation instance, which contains the materialized classes and objects.
	If this variable is nil, the assumptions is, that it is not installed.!

!BMPackage class methodsFor: 'system integration' stamp: 'lw 1/15/2013 23:00'!
add: aMorph toBinaryModuleSubmenuTo: aMenu
	"add the possibility to add a morph to the object list of a BMInstallation"
	
	| subMenu |
	subMenu := MenuMorph new defaultTarget: self.
	
	subMenu add: 'new module...' translated selector: #createNewInstallationWith: argument: aMorph.
	BMBundles select: [ :each | each isInstalled ] thenDo: [ :each |
		subMenu 
			add: each menuString 
			target: each installation
			selector: #addMorph: 
			argument: aMorph].
	
	aMenu 
		addLine;
		add: 'add to Binary Module...' translated subMenu: subMenu;
		addLine! !

!BMPackage class methodsFor: 'magic constants' stamp: 'WS 1/9/2013 17:41'!
bundleExtension
	^ '.bundle'! !

!BMPackage class methodsFor: 'system integration' stamp: 'lw 1/15/2013 23:21'!
createNewPackageWith: aMorph
	
	"Using #addMorph: instead of including the morph in the objects array, to enable adding classes to the classes array which are needed for the morph and its submorphs at only one point in BMInstallation."
	^(UIManager default request: 'Please type new module name' translated) 
		ifNotNil: [ :aName | 
			aName ifEmpty: [ nil ] ifNotEmpty: [
			((BMInstallation named: aName)
				addMorph: aMorph;
				pack) ] ]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 1/14/2013 11:06'!
fromFile
	"finds only files without extension and files ending with .bundle"
	| file |
	file := UIManager default chooseFileMatching: #('' 'bundle').
	file ifNotNil: [^ self fromFile: file ]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 23:18'!
fromFile: aFileName
	^ super new
		fromBinary: (BMBundle fromFile: aFileName);
		addToSystem;
		yourself! !

!BMPackage class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 23:11'!
fromInstallation: anInstallation
	^ super new
		bundle: anInstallation;
		addToSystem;
		yourself! !

!BMPackage class methodsFor: 'class initialization' stamp: 'WS 1/14/2013 11:05'!
initialize
	super initialize.
	BMBundles := OrderedCollection new! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:18'!
loadNamed: aPackageName
	^ self fromFile: aPackageName, self bundleExtension! !

!BMPackage class methodsFor: 'instance creation' stamp: 'lw 1/15/2013 23:22'!
named: aString
	
	^BMBundles 
		detect: [ :ea | ea name = aString asString ]
		ifNone: [ (UIManager default confirm: 'The bundle was not found. Do you want to create it?' translated) ifFalse: [^nil].
			(BMInstallation named: aString asString) pack
				addToSystem;
				yourself]! !

!BMPackage class methodsFor: 'instance creation' stamp: 'WS 1/9/2013 18:01'!
new
	self shouldNotImplement! !

!BMPackage methodsFor: 'package facade' stamp: 'lw 1/15/2013 22:58'!
addClass: aClass
	^self installation addClass: aClass! !

!BMPackage methodsFor: 'package facade' stamp: 'lw 1/15/2013 22:58'!
addObject: anObject
	^self installation addObject: anObject    ! !

!BMPackage methodsFor: 'utilities' stamp: 'lw 1/13/2013 19:43'!
addToSystem
	(BMBundles ifNil: [BMBundles := OrderedCollection new])
		addLast: self! !

!BMPackage methodsFor: 'accessing-derived' stamp: 'lw 1/15/2013 23:23'!
bundle
	"do we need to save the created file?"
	^bundle 
		ifNil: [BMBundle 
				named: name 
				classDescriptions: (self createClassDescriptionsFor: self installation classes) 
				serializedObjects: (FLSerializer serializeToByteArray: self installation objects) ]
		ifNotNil: [bundle isSymbol | bundle isString 
					ifTrue: [BMBundle fromFile: bundle]
					ifFalse: [bundle]]! !

!BMPackage methodsFor: 'creation' stamp: 'lw 1/15/2013 23:12'!
bundle: anInstallation
	
	installation := anInstallation.
	installation addDependent: self! !

!BMPackage methodsFor: 'utilities' stamp: 'WS 1/9/2013 18:00'!
createClassDescriptionsFor: aCollection
	^ aCollection collect: [ :class | BMClassDescription for: class ]! !

!BMPackage methodsFor: 'installation' stamp: 'lw 1/15/2013 23:23'!
createClasses
	self assert: bundle notNil.
	^ self bundle classDescriptions collect: [ :classDescription | classDescription install ]! !

!BMPackage methodsFor: 'saving' stamp: 'WS 1/9/2013 17:41'!
exportToFile
	^ self exportToFile: name, self class bundleExtension! !

!BMPackage methodsFor: 'saving' stamp: 'lw 1/15/2013 23:23'!
exportToFile: aFileName
	FLSerializer serialize: self bundle toFileNamed: aFileName.
	bundle := aFileName! !

!BMPackage methodsFor: 'saving' stamp: 'lw 1/14/2013 01:15'!
extractToFile: aFileName
	self exportToFile: aFileName.
	self uninstall! !

!BMPackage methodsFor: 'creation' stamp: 'lw 1/15/2013 23:23'!
fromBinary: aBMFile
	name := aBMFile name.
	bundle := aBMFile! !

!BMPackage methodsFor: 'installation' stamp: 'lw 1/15/2013 23:24'!
install
	self assert: bundle notNil.
	self assert: installation isNil.
	
	installation := BMInstallation
		of: self
		with: self createClasses
		and: self objects.
	installation addDependent: self.
	^installation! !

!BMPackage methodsFor: 'accessing-derived' stamp: 'lw 1/15/2013 22:58'!
installation
	^installation ifNil: [self notify: 'This package is not installed.' ]! !

!BMPackage methodsFor: 'testing' stamp: 'lw 1/15/2013 22:59'!
isInstalled
	^installation notNil! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/14/2013 11:02'!
menuString
	^ name! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name
	^ name! !

!BMPackage methodsFor: 'accessing' stamp: 'WS 1/9/2013 17:40'!
name: aString
	name := aString! !

!BMPackage methodsFor: 'installation' stamp: 'lw 1/15/2013 23:24'!
objects
	self assert: bundle notNil.
	^ FLMaterializer materializeFromByteArray: bundle serializedObjects! !

!BMPackage methodsFor: 'utilities' stamp: 'lw 1/14/2013 01:12'!
removeFromSystem
	BMBundles ifNotNil: 
		[BMBundles := BMBundles reject: [ :each | each == self]]! !

!BMPackage methodsFor: 'accessing-derived' stamp: 'lw 1/15/2013 23:24'!
serializedObjects
	^bundle 
		ifNil: [self error: 'Bundle not created from file.']
		ifNotNil: [ bundle serializedObjects ]  ! !

!BMPackage methodsFor: 'installation' stamp: 'lw 1/15/2013 23:24'!
uninstall
	"although unload depends heavily on data of Installation, for symmetrie to install, it is implemented here"
	self isInstalled 
		ifTrue: [ 
			bundle ifNil: [self notify: 'Are you sure? The classes and objects will be irrevocably removed.'].
			self installation classes do: [ :aClass | aClass removeFromSystem].
			"Only for morphs, we know how to remove them."
			self installation objects do: [ :each | each isMorph ifTrue: [each delete]].
			installation := nil]! !

Object subclass: #ExampleProject
	instanceVariableNames: 'foo bar'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules-Tests-Data'!

!ExampleProject methodsFor: 'as yet unclassified' stamp: 'RL 1/7/2013 16:29'!
foobar
	^ foo + bar! !

!ExampleProject methodsFor: 'initialization' stamp: 'RL 1/7/2013 16:29'!
initialize
	foo := 42.
	bar := 23.! !

TestCase subclass: #BMPackageTest
	instanceVariableNames: 'class bundle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BinaryModules-Tests'!

!BMPackageTest methodsFor: 'utility' stamp: 'WS 1/14/2013 15:49'!
compareClass: aClass with: anotherClass

	self 
		assert: aClass name == anotherClass name;
		assert: aClass methodDict size equals: anotherClass methodDict size;
		assert: aClass instVarNames equals: anotherClass instVarNames;
	 	assert: aClass category equals: anotherClass category;
	 	assert: aClass superclass == anotherClass superclass.
		
	aClass methodDict associationsDo: [ :assoc | self assert: (anotherClass methodDict at: assoc key) size 
														equals: assoc value size].
	self assert: aClass class methodDict size equals: anotherClass class methodDict size.
	aClass class methodDict associationsDo: [ :assoc | self assert: (anotherClass class methodDict at: assoc key) size 
														equals: assoc value size]! !

!BMPackageTest methodsFor: 'utility' stamp: 'lw 1/15/2013 22:52'!
serializeAndMaterializeSimpleClassDescription
	" destroys the content of the instance variable class "
	| aCopy aDescription aMaterializedClass |
	[aCopy := class copy. "To avoid comparing the class to itself."
	aDescription := BMClassDescription for: class.
	(Smalltalk at: class name ifAbsent: nil) ifNotNil: [ :obsoleteClass | obsoleteClass removeFromSystem].
	aMaterializedClass := aDescription install.
	
	self compareClass: aCopy with: aMaterializedClass.
	self deny: aCopy == aMaterializedClass.]
		ensure: [aMaterializedClass ifNotNil: [
					self tearDownClass: aMaterializedClass]].! !

!BMPackageTest methodsFor: 'test graph' stamp: 'WS 1/14/2013 15:49'!
setupComplexTestClass
	
	class := Object subclass: #BMTestClass
		instanceVariableNames: 'lorem'
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules-Tests-Data'.
		
	class compile: 'ipsum ^true'.
	class compile: 'dolor ^false'.
	class class compile: 'sit: amet amet isNil. ^Object new'.

	^ class! !

!BMPackageTest methodsFor: 'test graph' stamp: 'WS 1/14/2013 15:49'!
setupTestClass
	
	class := Object subclass: #BMTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules-Tests-Data'.
		
	class compile: 'foo ^true'.
	class compile: 'bar ^false'.

	^ class! !

!BMPackageTest methodsFor: 'test graph' stamp: 'lw 1/15/2013 22:52'!
simpleTestClass
	
	class := Object subclass: #BMSimpleTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'BinaryModules-Tests-Data'.
	
	^ class! !

!BMPackageTest methodsFor: 'running' stamp: 'WS 1/14/2013 15:49'!
tearDown

	super tearDown.
	class ifNotNil: [class removeFromSystem].
	bundle ifNotNil: [bundle removeFromSystem]! !

!BMPackageTest methodsFor: 'utility' stamp: 'WS 1/14/2013 15:49'!
tearDownClass: testClass
	"testClass superclass removeSubclass: testClass"
	
	testClass removeFromSystem! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 1/14/2013 15:49'!
testClassDescriptionCreation

	|aDescription|

	aDescription := BMClassDescription for: ExampleProject.
	self assert: aDescription name equals: #ExampleProject.
	self assert: (aDescription represents: ExampleProject).
! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/15/2013 22:54'!
testCreatePackage
	
	| aPackage |
	self setupTestClass.
	aPackage := BMInstallation 
						named: 'testPackage' 
						withClasses: {class} 
						andObjects: {class new}.
	self assert: aPackage classes first equals: class.
	self assert: aPackage objects first class equals: class! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/15/2013 22:57'!
testDeserializePackage
	
	| aClassCopy  aPackage |
	self setupTestClass.
	((BMInstallation
			named: 'testPackage' 
			withClasses: {class} 
			andObjects: {class new}) 
		exportToFile: 'testPackage') 
			removeFromSystem.
	aClassCopy := class copy.
	class removeFromSystem.
	bundle := BMPackage fromFile: 'testPackage'.
	self 
		should: [FLMaterializer materializeFromByteArray: bundle serializedObjects] 
		raise: FLClassNotFound.
	aPackage := bundle install.
	self compareClass: aClassCopy with: (Smalltalk at: aPackage classes first name).
	self assert: aPackage objects first class equals: (Smalltalk at: aPackage classes first name).! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 1/14/2013 15:48'!
testMaterializeClassDescription
	class := self setupTestClass.
	self serializeAndMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'WS 1/14/2013 15:48'!
testMaterializeComplexClassDescription
	"self new testMaterializeComplexClassDescription"
	class := self setupComplexTestClass.
	self serializeAndMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/15/2013 22:51'!
testMaterializeSimpleClassDescription
	class := self simpleTestClass.
	self serializeAndMaterializeSimpleClassDescription! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/15/2013 22:54'!
testSerializePackage
	
	| aClass |
	aClass := self setupTestClass.
	bundle := (BMInstallation 
			named: 'testPackage' 
			withClasses: {aClass} 
			andObjects: {aClass new}) 
		exportToFile: 'testPackage'.
	self assert: (FileDirectory default fileExists: 'testPackage').
	FileDirectory default deleteFileNamed: 'testPackage'.
	self deny: (FileDirectory default fileExists: 'testPackage').
	aClass removeFromSystem! !

!BMPackageTest methodsFor: 'testing' stamp: 'lw 1/15/2013 23:22'!
testUninstallPackage
	
	self setupTestClass.
	bundle := (BMInstallation 
				named: 'testPackage' 
				withClasses: {class} 
				andObjects: {class new}) pack.
	self assert: bundle isInstalled.
	self assert: class allInstances size equals: 1.
	[bundle uninstall] on: Warning do: [ :warning | warning resume ].
	Smalltalk garbageCollect.
	self deny: bundle isInstalled.
	self assert: class isObsolete.
	self assert: class allInstances size equals: 0.! !
BMPackage initialize!
